==> evolvotron-0.3.1.tar.gz: evolvotron/libevolvotron/transform.h <==
// Source file for evolvotron
// Copyright (C) 2002,2003 Tim Day
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*! \file 
  \brief Interface for class Transform.
*/

#ifndef _transform_h_
#define _transform_h_

#include "useful.h"
#include "xyz.h"
#include <vector>

//! Class representing 3d linear transforms.
/*! Not much functionality currently because is used mainly to pass info around for warp functionality
 */
class Transform 
{
 public:
  //! Default constructor sets up identity.
  Transform();

  //! Copy constructor.
  Transform(const Transform&);

  //! Constructor specifying column vectors.
  Transform(const XYZ& t,const XYZ& x,const XYZ& y,const XYZ& z);

  //! Constructor specifying column-wise elements.
  Transform(const std::vector<float>& v);

  //! virtual destructor in case of extension
  virtual ~Transform();

  //@{
  //! Accessor
  const XYZ& translate() const
    {
      return _translate;
    }
  const XYZ& basis_x() const
    {
      return _basis_x;
    }
  const XYZ& basis_y() const    
    {
      return _basis_y;
    }
  const XYZ& basis_z() const    
    {
      return _basis_z;
    }

  void translate(const XYZ &t)
     {
       _translate=t;
     }
  void basis_x(const XYZ &x)
     {
       _basis_x=x;
     }
  void basis_y(const XYZ &y)
     {
       _basis_y=y;
     }
  void basis_z(const XYZ &z)
     {
       _basis_z=z;
     }
  //@}

  //! Get column-wise element values as a vector
  const std::vector<float> get_columns() const;

  //! Transform a point
  const XYZ transformed(const XYZ& p) const;

  //! Transform a point with no translation
  const XYZ transformed_no_translate(const XYZ& p) const;

  //! Concatenate transforms
  Transform& concatenate_on_right(const Transform& t);

  //! Concatenate transforms
  Transform& concatenate_on_left(const Transform& t);

 protected:
  //@{
  //! Translation component (column vector in matrix).
  XYZ _translate;
  XYZ _basis_x;
  XYZ _basis_y;
  XYZ _basis_z;
};

inline const XYZ operator*(const Transform& t,const XYZ& p)
{
  return 
     t.basis_x()*p.x()
    +t.basis_y()*p.y()
    +t.basis_z()*p.z()
    +t.translate();
}

inline std::ostream& operator<<(std::ostream& out,const Transform& t)
{
  return out << t.translate() << ";" << t.basis_x() << "," << t.basis_y() << "," << t.basis_z();
}

class TransformRotateX : public Transform
{
 public:
  TransformRotateX(float a)
    {
      const float sa=sin(a);
      const float ca=cos(a);
      
      basis_x(XYZ(1.0f,0.0f,0.0f));
      basis_y(XYZ(0.0f, ca , sa ));
      basis_z(XYZ(0.0f,-sa , ca ));
    }
};

class TransformRotateY : public Transform
{
 public:
  TransformRotateY(float a)
    {
      const float sa=sin(a);
      const float ca=cos(a);
      
      basis_x(XYZ( ca ,0.0f,-sa ));
      basis_y(XYZ(0.0f,1.0f,0.0f));
      basis_z(XYZ( sa ,0.0f, ca ));
    }
};

class TransformRotateZ : public Transform
{
 public:
  TransformRotateZ(float a)
    {
      const float sa=sin(a);
      const float ca=cos(a);
      
      basis_x(XYZ( ca , sa ,0.0f));
      basis_y(XYZ(-sa , ca ,0.0f));
      basis_z(XYZ(0.0f,0.0f,1.0f));
    }
};

#endif

==> evolvotron-0.4.0.tar.gz: evolvotron/libevolvotron/transform.h <==
// Source file for evolvotron
// Copyright (C) 2002,2003 Tim Day
/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*! \file 
  \brief Interface for class Transform.
*/

#ifndef _transform_h_
#define _transform_h_

#include "useful.h"
#include "xyz.h"
#include <vector>

//! Class representing 3d linear transforms.
/*! Not much functionality currently because is used mainly to pass info around for warp functionality
 */
class Transform 
{
 public:
  //! Default constructor sets up identity.
  Transform();

  //! Copy constructor.
  Transform(const Transform&);

  //! Constructor specifying column vectors.
  Transform(const XYZ& t,const XYZ& x,const XYZ& y,const XYZ& z);

  //! Constructor specifying column-wise elements.
  Transform(const std::vector<real>& v);

  //! virtual destructor in case of extension
  virtual ~Transform();

  //@{
  //! Accessor
  const XYZ& translate() const
    {
      return _translate;
    }
  const XYZ& basis_x() const
    {
      return _basis_x;
    }
  const XYZ& basis_y() const    
    {
      return _basis_y;
    }
  const XYZ& basis_z() const    
    {
      return _basis_z;
    }

  void translate(const XYZ &t)
     {
       _translate=t;
     }
  void basis_x(const XYZ &x)
     {
       _basis_x=x;
     }
  void basis_y(const XYZ &y)
     {
       _basis_y=y;
     }
  void basis_z(const XYZ &z)
     {
       _basis_z=z;
     }
  //@}

  //! Get column-wise element values as a vector
  const std::vector<real> get_columns() const;

  //! Transform a point
  const XYZ transformed(const XYZ& p) const;

  //! Transform a point with no translation
  const XYZ transformed_no_translate(const XYZ& p) const;

  //! Concatenate transforms
  Transform& concatenate_on_right(const Transform& t);

  //! Concatenate transforms
  Transform& concatenate_on_left(const Transform& t);

 protected:
  //@{
  //! Translation component (column vector in matrix).
  XYZ _translate;
  XYZ _basis_x;
  XYZ _basis_y;
  XYZ _basis_z;
};

inline const XYZ operator*(const Transform& t,const XYZ& p)
{
  return 
     t.basis_x()*p.x()
    +t.basis_y()*p.y()
    +t.basis_z()*p.z()
    +t.translate();
}

inline std::ostream& operator<<(std::ostream& out,const Transform& t)
{
  return out << t.translate() << ";" << t.basis_x() << "," << t.basis_y() << "," << t.basis_z();
}

class TransformRotateX : public Transform
{
 public:
  TransformRotateX(real a)
    {
      const real sa=sin(a);
      const real ca=cos(a);
      
      basis_x(XYZ(1.0,0.0,0.0));
      basis_y(XYZ(0.0, ca , sa ));
      basis_z(XYZ(0.0,-sa , ca ));
    }
};

class TransformRotateY : public Transform
{
 public:
  TransformRotateY(real a)
    {
      const real sa=sin(a);
      const real ca=cos(a);
      
      basis_x(XYZ( ca ,0.0,-sa ));
      basis_y(XYZ(0.0,1.0,0.0));
      basis_z(XYZ( sa ,0.0, ca ));
    }
};

class TransformRotateZ : public Transform
{
 public:
  TransformRotateZ(real a)
    {
      const real sa=sin(a);
      const real ca=cos(a);
      
      basis_x(XYZ( ca , sa ,0.0));
      basis_y(XYZ(-sa , ca ,0.0));
      basis_z(XYZ(0.0,0.0,1.0));
    }
};

#endif
